/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import java.util.*;
import org.w3c.dom.Node;


/**
 * Children (besides the inherited):<br>
 *   ignore (this enables no-selection in the single lists - radiolist and popuplist)<br>
 *   option, option...<br>
 */
public abstract class A_SingleValueList extends A_SingleValueField {
	
	private List options;
    private String ignore;
    
	public A_SingleValueList(Form form) {
		super(form);
	}
	
	public A_SingleValueList(Form form, Node field) {
		super(form, field);
	}
		
	/**
	 * Look for relevant sub-elements
	 * @param child
	 * @param childType
	 * @param childName
	 * @return true if child is considered handled, false if it didn't cause any changes
	 */
	public boolean handleChildNode(Node child, short childType, String childName)
	{
		if(super.handleChildNode(child, childType, childName))
			return true;
		if(childType == Node.ELEMENT_NODE) {
			if (childName.equals(Form.FBTAG_OPTION)) {
				addOption(new Option(form, child));
				return true;
			}
			else if (childName.equals(Form.FBTAG_IGNORE)) {
				setIgnore(Form.collectText(child));
				return true;
			}
		}
		return false;
	}
	
	//
	// accessors
	//
	
	public List getOptions() {
		return options;
	}
	
	public void setOptions(List opts) {
		options = opts;
	}
	
	public void addOption(Option opt) {
		if(options == null)
			options = new ArrayList();
		options.add(opt);
	}
	
	public void removeOption(Option opt) {
		if(options != null)
			options.remove(opt);
	}
	
	public String getIgnore() {
		return this.ignore;
	}
	
	public void setIgnore(String str) {
		ignore = str;
		if(ignore != null && ignore.equals(getValue()))
			super.setValue(null); // ignore value
	}
	
	public void setValue(String val) {
		if(val != null && val.equals(getIgnore()))
			super.setValue(null); // ignore value
		else
			super.setValue(val);
	}
	
	//
	//
	//
	
	public String childrenToXML(String indent, boolean inclVal) 
	{
		String tmp = super.childrenToXML(indent, inclVal);
		if(tmp == null && getValue() == null)
			return null;
		StringBuffer b = new StringBuffer();
		if(tmp != null)
			b.append(tmp);
		if(getIgnore() != null)
			b.append("\n" + indent + "<"+Form.FBTAG_IGNORE+">" + getIgnore() + "</"+Form.FBTAG_IGNORE+">");
		if(getOptions() != null) {
			for(int i = 0; i < getOptions().size(); i++) {
				Option opt = (Option)getOptions().get(i);
				b.append(opt.toXML(indent));
			}
		}
		return b.toString();
	}
}