/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import org.w3c.dom.*;
import java.text.*;

public class FieldFormat 
{
	private String pattern;
	private String type;
	private Format formatter;
	private Form form;

	public FieldFormat(Form form) {
		super();
		this.form = form;
	}
	
	public FieldFormat(Form form, Node format) {
		super();
		this.form = form;
		populateFromNode(format);
	}
	
	public void populateFromNode(Node node) 
	{
		NodeList list = node.getChildNodes();
		int len = list.getLength();
		for(int i = 0; i < len; i++) {
			Node c = list.item(i);
			if(c.getNodeType() == Node.ELEMENT_NODE){
				if(Form.FBTAG_TYPE.equals(c.getNodeName()))
					setType(Form.collectText(c,""));
				else if(Form.FBTAG_PATTERN.equals(c.getNodeName()))
					setPattern(Form.collectText(c,""));
			}
		}
	}	
	
	//
	//
	//
	
	public String getType() {
		return type;
	}
	
	public void setType(String type) {
		this.type = type;
		if(type==null)
			formatter = null;
	}

	public String getPattern() {
		return pattern;
	}
	
	public void setPattern(String pattern) {
		this.pattern = pattern;
	}

	public Format formatter() {
		if(formatter == null) {
			if("date".equals(type)) {
				if(getPattern()==null)
					formatter = new SimpleDateFormat("yyMMdd-HHmmss",form.locale());
				else
					formatter = new SimpleDateFormat(getPattern(),form.locale());
			}
			else if("number".equals(type)) {
				formatter = NumberFormat.getInstance(form.locale());
			}
			else if("integer".equals(type)) {
				formatter = NumberFormat.getIntegerInstance(form.locale());
			}
			else if("currency".equals(type)) {
				formatter = NumberFormat.getCurrencyInstance(form.locale());
			}
			else if("percent".equals(type)) {
				formatter = NumberFormat.getPercentInstance(form.locale());
			}
			else if("decimal".equals(type)) {
				if(getPattern()==null)
					formatter = new DecimalFormat();
				else
					formatter = new DecimalFormat(getPattern());
			}
			else {
				formatter = null;
			}
		}
		return formatter;
	}
	
	//
	//
	//
	
	public String toXML(String indent) {
		return "\n" + indent + "<"+Form.FBTAG_FORMAT+">" + childrenToXML(indent + Form.PP_INDENT) + "\n" + indent + "</"+Form.FBTAG_FORMAT+">";
	}
	
	public String childrenToXML(String indent) 
	{
		StringBuffer b = new StringBuffer();
		if(getType() != null)
			b.append("\n" + indent + "<"+Form.FBTAG_TYPE+">" + getType() + "</"+Form.FBTAG_TYPE+">");
		if(getPattern() != null)
			b.append("\n" + indent + "<"+Form.FBTAG_LABEL+">" + getPattern() + "</"+Form.FBTAG_LABEL+">");
		return b.toString();
	}
}
