/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import org.w3c.dom.Node;


import java.util.*;

/**
 * <p>&lt;options&gt;</p>
 * 
 * Check that at least minimum (default 1) and no more than maximum (default is no limit) values are selected.<br> 
 * Furthermore makes sure that the values are actually among the options - mostly a security precautions
 * towards hackers that could fiddle with the HTML.<br>
 * If minimum is 0 (zero) an empty selection is allowed.<br>
 * 
 * Children:<br>
 *   -error<br>
 *   -min<br>
 *   -max<br>
 *
 */
public class OptionsValidator extends A_RangeValidator {
	
	public OptionsValidator(Form form) {
		super(form);
	}
	
	public OptionsValidator(Form form, Node node) {
		super(form, node);
	}
	
	public boolean validate(I_Field field) {
		int count = 0;
		List allVals = field.getValues();
		if(allVals != null)
			count = allVals.size();
		int minimum = 1; // Default value when not set, i.e. perfect for radio and popup lists
		if(getMin() != null && getMin().length() > 0)
			minimum = Integer.parseInt(getMin());
		if(count < minimum)
			return false;
		// Default for max is no limit, so we won't check if not specified
		if(getMax() != null && getMax().length() > 0) {
			int maximum = Integer.parseInt(getMax());
			if(count > maximum)
				return false;
		}
		if(count == 0)
			return true; // no selection to check

		Iterator vals = allVals.iterator();
		while(vals.hasNext()) {
			String val = (String)vals.next();
			Iterator opts = field.getOptions().iterator();
			boolean match = false;
			while(opts.hasNext()) {
				String optval = ((Option)opts.next()).getValue();
				if(val.equals(optval))
					match = true;
			}
			if(!match)
				return false;
		}
		
		return true;
	}
}

