/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import java.util.*;
import org.w3c.dom.*;

/**
 * &lt;raw&gt;<br>
 * No children - just text or CDATA<br>
 */
public class RawField implements I_Field {
	
	private Form form;
	private String raw;

	public RawField(Form form) {
		super();
		this.form = form;
	}
	
	public RawField(Form form, Node field) {
		super();
		this.form = form;
		this.populateFromNode(field);
	}

	public int cardinality() {
		return I_Field.CARDINALITY_NONE;
	}
	
	public boolean isReadOnly() {
		return true;
	}
	
    public int validate() {
    	return 0;
    }
	
	public String tagName() {
		return Form.FBTAG_RAW;
	}
	
	public Form form() {
		return this.form;
	}
	
	public List getErrorMessages() {
		return null;
	}
	public void clearErrorMessages() {
		return;
	}
	public void addErrorMessage(String msg) {
		return;
	}
	
	public void populateFromNode(Node field) 
	{

		NodeList children = field.getChildNodes();
		int len = children.getLength();
		for(int i = 0; i < len; i++) {
			Node child = children.item(i);
			short ct = child.getNodeType();
			if(ct == Node.CDATA_SECTION_NODE || ct == Node.TEXT_NODE) {
				if(raw == null)
					raw = child.getNodeValue();
				else
					raw = raw + child.getNodeValue();
			}
		}
	}
	
	//
	// accessors
	//
	
	public String getRaw() {
		return raw;
	}
	
	public void setRaw(String str) {
		this.raw = str;
	}

	public String getName() {
		return null;
	}
	
	public String getIdent() {
		return null;
	}

	public String getValue() {
		return  null;
	}
	public void setValue(String value) {
		return;
	}
	public List getValues() {
		return  null;
	}
	public void setValues(List values) {
		return;
	}
	public void setValues(String[] values) {
		return;
	}

	public List getOptions(){
		return null;
	}
	
	public void setOptions(List opts) {
		return;
	}
	
	public FieldFormat getFormat(){
		return null;
	}

	//
	//
	//
	
	public String toString() {
		return toXML("",false);
	}

	public String toXML(String indent, boolean inclVal) {
		if(getRaw() != null)
			return "\n" + indent + "<"+tagName()+">" + Form.cdata(getRaw()) + (getRaw().endsWith("\n") ? indent : "") + "</"+tagName()+">";
		else
			return "\n" + indent + "<"+tagName()+" />";
	}
	
	public String toHTML() {
		return "\n" + getRaw();
	}
}

