/*
 * Copyright (c) 2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision$
 * $LastChangedDate$
 * 
 */

package com.codedroids.http;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.methods.GetMethod;

public class HttpClientUtils {

	/**
	 * Loads the contents from the given <code>uri</code>. If HttpStatus is OK, then the contents is returned,
	 * otherwise an exception with HTTP status code is returned.
	 * @param uri
	 * @return
	 * @throws Exception
	 */
	public static String httpGet(String uri) throws Exception {
		return httpGet(uri, null, null, null, false);
	}
	
	/**
	 * Loads the contents from the given <code>uri</code>. If HttpStatus is OK, then the contents is returned,
	 * otherwise an exception with HTTP status code is returned.
	 * 
	 * <p>Performs Basic authentication with the provided credentials if challenged by server. 
	 * All authentication parameters must be not null in order for it to be enabled  
	 * 
	 * @param uri
	 * @param user User name for <code>realm</code>, required when using authentication.
	 * @param passwd Password (clear text) for <code>realm</code>, required when using authentication.
	 * @param realm Provides the realm (on the <code>host</code>), optional - defaults to "/".
	 * @param preemptive Determines if authentication is preemptive (if true credentials are sendt even if not challenged by server). 
	 *                   If not authentication mode this parameter is ignored.
	 * @return
	 * @throws Exception
	 */
	public static String httpGet(String uri, String user, String passwd, String realm, boolean preemptive) throws Exception {
		String retval;
		HttpClient client = new HttpClient();
		GetMethod get = new GetMethod(uri);		
		// If all authorization related params are present, prepare for authentication mode
		if(user != null && passwd != null) {
			String host = get.getURI().getHost();
			//int port = get.getURI().getPort();
			if(realm==null)
				realm="/";
			UsernamePasswordCredentials creds = new UsernamePasswordCredentials(user, passwd);
			/* This is the httpclient 3 way of doing it but as OC 6.0.x came with httpclient 2 the deprecated methods below ared used for now 
			AuthScope scope = new AuthScope(host, port, realm, "Basic");
			client.getState().setCredentials(scope, creds);
			client.getParams().setAuthenticationPreemptive(true); */
			client.getState().setCredentials(realm, host, creds);
			client.getState().setAuthenticationPreemptive(preemptive);
			get.setDoAuthentication(true); 
		}
		try {
			int status = client.executeMethod(get);
			if(status == HttpStatus.SC_OK)
				retval = get.getResponseBodyAsString();
			else
				throw new Exception ("HttpClient status = " + status);
		} finally {
			get.releaseConnection();
		}
		return retval;
	}
	
}
