/*
 * Copyright (c) 2004-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 180 $
 * $LastChangedDate: 2007-06-20 11:22:10 +0200 (Wed, 20 Jun 2007) $
 * 
 */

package com.codedroids.j;

import java.util.Map;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

public class ListUtils {

	private static class LUEntry implements Map.Entry {
		private Object key;
		private Object value;
		public LUEntry(Object key, Object value) {
			this.key = key;
			this.value = value;
		}
		public Object getKey() { return key; }
		public Object getValue() { return value; }
		public Object setValue(Object ignored) { /* No Op. */ return value; } 
	}
	
	/**
	 * Generates list with Map.Entry implementing objects from a string like "foo:bar;more:less;"
	 * @param list
	 * @param source
	 * @return
	 */
	public static List populateListWithMapEntriesFromString(List list, String source)
	{
		String[] a = source.split(";");
		for(int i = 0; i<a.length; i++) {
			String[] b = a[i].split(":");
			String k = b[0];
			String v = (b.length == 1) ? null : b[1];
			list.add(new LUEntry(k, v));
		}
		return list;
	}
	
	public static List listOfMapEntriesWithKey(List list, String key)
	{
		ArrayList retval = new ArrayList();
		for(int i=0; i<list.size(); i++) {
			String s = (String)((Map.Entry)list.get(i)).getKey();
			if((s == null && key == null) || (s != null && s.equals(key))) {
				retval.add(list.get(i));
			}
		}
		return retval;
	}
	
	public static boolean listOfMapEntriesContainsKey(List list, String key) {
		for(int i=0; i<list.size(); i++) {
			String s = (String)((Map.Entry)list.get(i)).getKey();
			if((s == null && key == null) || (s != null && s.equals(key))) {
				return true;
			}
		}
		return false;
	}
	
	public static Map.Entry findMapEntryWithKey(List list, String key)
	{
		for(int i=0; i<list.size(); i++) {
			String s = (String)((Map.Entry)list.get(i)).getKey();
			if((s == null && key == null) || (s != null && s.equals(key))) {
				return (Map.Entry)list.get(i);
			}
		}
		return null;
	}

	public static Map findMapWithKeyValue(List list, Object key, Object value)
	{
		for(int i=0; i<list.size(); i++) {
			Map m = (Map)list.get(i);
			Object v = m.get(key);
			if((v == null && value == null) || (v != null && v.equals(value))) {
				return m;
			}
		}
		return null;
	}

	/**
	 * Combines the entries in the list into a string by means of toString. Entries are separated
	 * by the given separator (defaults to empty string). If lastSeparator is given it is used between
	 * the last 2 entries (ie. "one, two, three, and four) where separator=", " and lastSeparator=", and ".
	 * @param list
	 * @param separator
	 * @param lastSeparator
	 * @return
	 */
	public static String concat(List list, String separator, String lastSeparator) 
	{
		if(list == null)
			return null;
		StringBuffer b = new StringBuffer();
		if(separator==null)
			separator = "";
		if(lastSeparator==null)
			lastSeparator = separator;
		int counter = 0, last = list.size()-1;
		Iterator i = list.iterator();
		while(i.hasNext()) {
			if(counter > 0) {
				if(counter == last)
					b.append(lastSeparator);
				else
					b.append(separator);
			}
			b.append(i.next().toString());
			counter++;
		}
		return b.toString();
	}
	public static String concat(String[] list, String separator, String lastSeparator) 
	{
		if(list == null)
			return null;
		StringBuffer b = new StringBuffer();
		if(separator==null)
			separator = "";
		if(lastSeparator==null)
			lastSeparator = separator;
		int counter = 0, last = list.length-1;
		for(int i = 0; i<=last; i++) {
			if(counter > 0) {
				if(counter == last)
					b.append(lastSeparator);
				else
					b.append(separator);
			}
			b.append(list[i]);
			counter++;
		}
		return b.toString();
	}
	
	/**
	 * Fills a given List with values from the given array
	 * @param list
	 * @param array
	 * @param offset - start copy process from here
	 * @param step - and step through the array with this increment size
	 * @return
	 */
	public static List fill(List list, Object[] array, int offset, int step) 
	{
		for (int i=offset; i<array.length; i+=step) {
			list.add(array[i]);
		}
		return list;
	}
}
