/*
 * Copyright (c) 2003-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 180 $
 * $LastChangedDate: 2007-06-20 11:22:10 +0200 (Wed, 20 Jun 2007) $
 * 
 */

package com.codedroids.j;

import java.util.Iterator;
import java.util.Map;
import java.security.*;


public class StringUtils {

	/**
	 * 
	 * @param source
	 * @param escapeThese
	 * @param withThis
	 * @return
	 */
	public static String escape(String source, String escapeThese, String withThis)
	{
		int l = source.length();
		StringBuffer buf = new StringBuffer(l+10);
		for(int i=0; i<l; i++) {
			char ch = source.charAt(i);
			if(escapeThese.indexOf(ch) != -1)
				buf.append(withThis);
			buf.append(ch);
		}
		return buf.toString();
	}
	
	public static String s4n(String obj) {
		return (obj == null ? "" : obj);
	}

	public static String s4n(String obj, String def) {
		return (obj == null ? def : obj);
	}
	
	public static String n4s(String obj) {
		return ((obj == null || obj.trim().equals("")) ? null : obj);
	}
	
	public static String padLeft(String val, int width, char padding) {
		StringBuffer b = new StringBuffer(width);
		b.append(val);
		for(int i=val.length(); i<width; i++)
			b.append(padding);
		return b.toString();
	}
	
	public static String padRight(String val, int width, char padding) {
		StringBuffer b = new StringBuffer(width);
		for(int i=val.length(); i<width; i++)
			b.append(padding);
		b.append(val);
		return b.toString();
	}
	
	public static String padCenter(String val, int width, char padding) {
		StringBuffer b = new StringBuffer(width);
		int l = (width - val.length()) / 2; // rounded down which means that we favor: "-aa--" over "--aa-"
		int r = width - val.length() - l;
		for(int i=0; i<l; i++)
			b.append(padding);
		b.append(val);
		for(int i=0; i<r; i++)
			b.append(padding);
		return b.toString();
	}
	
	public static String pad(String val, int width, char padding, String dir) {
		if("L".equals(dir))
			return padLeft(val, width, padding);
		else if("R".equals(dir))
			return padRight(val, width, padding);
		else if("C".equals(dir))
			return padCenter(val, width, padding);
		else
			return val;
	}
	

	public static String replaceMacroStrings(String str, Map params) 
	{
	    return replaceStrings(str, params, "${", "}");
	}
	public static String replaceStrings(String str, Map params) 
	{
	    return replaceStrings(str, params, "", "");
	}	
	public static String replaceStrings(String str, Map params, String open, String close) 
	{
		if(str==null)
			return null;
		if(params == null || params.size()==0)
			return str;
		
	    StringBuffer buffer = new StringBuffer(str);
	    Iterator it = params.entrySet().iterator();
	    while(it.hasNext()) {
			Map.Entry entry = (Map.Entry)it.next();
			String macro = open + entry.getKey() + close;
	    	int ndx = buffer.indexOf(macro);
	    	if (ndx == -1)
	    		continue;
	    	int macroLen = macro.length();
	    	String sub = (String)entry.getValue();
			if(sub == null)
				sub = ""; // assumed that it is null on purpose
	    	int subLen = sub.length();
	    	while (ndx != -1) {
	 	       buffer.replace(ndx, ndx+macroLen, sub);
	 	       ndx = buffer.indexOf(macro, ndx+subLen);
			}
	    }
	    return buffer.toString();
	}
	
	private static final char hexChars[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
	private static SecureRandom random;
	/**
	 * Returns a hex encoded string representing the hashLength bytes (i.e. twice the hashLength)
	 * @param hashLength specifies number of bytes
	 * @return 
	 */
	public static String randomString(int hashLength) {
		try {
			if(random == null) {
				random = SecureRandom.getInstance("SHA1PRNG");
				random.setSeed(java.lang.System.currentTimeMillis());
			}
			byte bytes[] = new byte[hashLength];
			random.nextBytes(bytes);
			StringBuffer buf = new StringBuffer(hashLength*2);
			for(int i = 0; i<hashLength; i++) {
				int b = 128+(int)bytes[i];
				buf.append(hexChars[b/16]);
				buf.append(hexChars[b%16]);
			}
			return buf.toString();		
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public static String abbreviate(String src, int max, String ellipses) {
		int elen;
		if(ellipses==null || (elen = ellipses.length())==0)
			throw new IllegalArgumentException("Ellipses string must be given ("+ellipses+")");
		if(Math.abs(max) < ellipses.length())
			throw new IllegalArgumentException("Max. absolute value ("+max+") must be at least 3 characters");
		if(src != null && src.length() > Math.abs(max)) {
			if(max > 0)
				return src.substring(0, max-elen)+ellipses;
			else
				return ellipses+src.substring(src.length()+max+elen);
		} else {
			return src;
		}
	}
}
