/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.jam;

import javax.servlet.http.HttpSession;
import java.util.HashMap;

public class JspAppMachineFactory {
	
	/**
	 * Keep one instance per session
	 * @return
	 */
	public static synchronized JspAppMachineFactory getFactory(HttpSession session) {
		synchronized(session) {
			JspAppMachineFactory factory = (JspAppMachineFactory)session.getAttribute("com.codedroids.jam");
			if(factory == null) {
				factory = new JspAppMachineFactory();
				session.setAttribute("com.codedroids.jam", factory);
			}
			return factory;
		}
	}
	
	//
	// Instance
	//
	
	private int instanceCount = 0; // bit simplistic approach for uniqueness
	private HashMap instances = new HashMap();
	
	public JspAppMachineFactory() {
		super();
	}
	
	
	/**
	 * Looks up existing instances of jam's for the given app name in the given session
	 * @param jamName
	 * @param jamId
	 * @param session
	 * @return null if the instance is unknown.
	 */
	public JspAppMachine getJamInstance(String jamId) {
		synchronized(instances) {
			return (JspAppMachine)instances.get(jamId);
		}
	}
	
	public JspAppMachine newJamInstance(JspApp app) {
		return newJamInstance(app, app.getName());
	}
	public JspAppMachine newJamInstance(JspApp app, String jamName) {
		JspAppMachine jam;
		synchronized(instances) {
			String jamId = jamName + "-" + instanceCount++;
			jam = new JspAppMachine(jamId, app); 
			instances.put(jamId, jam);
		}
		return jam;
	}

}
