/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.jam;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class JspAppState implements JspAppNode {

	//
	// Instance
	//
	private JspApp parentApp;
	private String name;
	private String process;
	private String display;
	private List transitions;
	
	// 
	// Constructors
	//

	public JspAppState(JspApp parent) {
		super();
		this.parentApp = parent;
	}
	
	public JspApp jspApp() {
		return parentApp;
	}
	
	//
	// Accessors
	//
	
	public String getName() {
		return name;
	}

	public void setName(String str) {
		this.name = str;
	}
	
	public String getDisplay() {
		return display;
	}

	public void setDisplay(String str) {
		this.display = str;
	}

	public String getProcess() {
		return process;
	}

	public void setProcess(String str) {
		this.process = str;
	}

	public List getTransitions() {
		return transitions;
	}

	public void setTransitions(List transitions) {
		this.transitions = transitions;
	}
	
	public void addTransition(JspAppTransition t) {
		if(this.transitions == null)
			this.transitions = new ArrayList();
		this.transitions.add(t);
	}
	
	public void removeTransition(JspAppTransition t) {
		if(this.transitions != null)
			this.transitions.remove(t);
	}
	
	//
	// Stuff
	//
	
	public String toXML(String indent) {
		StringBuffer b = new StringBuffer();
		b.append("\n"+indent+"<"+JAM_STATE_TAG+">");
		b.append("\n"+indent+"<"+JAM_STATE_TAG+" "+JAM_NAME_ATTR+"=\""+getName()+"\">");
		b.append("\n"+indent+PP_INDENT+"<"+JAM_DISPLAY_TAG+">" + getDisplay() + "</"+JAM_DISPLAY_TAG+">");
		if(getProcess() != null)
		  b.append("\n"+indent+PP_INDENT+"<"+JAM_PROCESS_TAG+">" + getProcess() + "</"+JAM_PROCESS_TAG+">");
		if(getTransitions() != null) {
			b.append("\n"+indent+PP_INDENT+"<"+JAM_TRANSITIONS_TAG+">");
			Iterator it = getTransitions().iterator();
			while(it.hasNext()){
				JspAppTransition f = (JspAppTransition)it.next();
				b.append(f.toXML(indent+PP_INDENT+PP_INDENT));
			}
			b.append("\n"+indent+PP_INDENT+"</"+JAM_TRANSITIONS_TAG+">");
		}
		b.append("\n"+indent+"</"+JAM_STATE_TAG+">");
		return b.toString();
	}

	//
	// Parsing
	//
	
	public JspAppNode parseNode(Node root)
	{
		NamedNodeMap attrs = root.getAttributes();
		for(int i=0; i<attrs.getLength(); i++) {
			Node attr = attrs.item(i);
			String an = attr.getNodeName();
			if(an.equals(JAM_NAME_ATTR)) {
				setName(JspApp.collectText(attr));
			}
		}		Node child = root.getFirstChild();
		//String initStateName = null;
		while( child != null) {
			short nt = child.getNodeType();
			switch(nt) {
				case Node.ELEMENT_NODE:
					String nn = child.getNodeName();
					if (nn.equals(JAM_DISPLAY_TAG)) {
						setDisplay(JspApp.collectText(child));
					}
					else if (nn.equals(JAM_PROCESS_TAG)) {
						setProcess(JspApp.collectText(child));
					}
					else if (nn.equals(JAM_TRANSITIONS_TAG)) {
						parseTransitions(child);
					}
					break;
				default:		
					break;
			}
			child = child.getNextSibling();
		}
		return this;
	}
	
	protected void parseTransitions(Node transitions)
	{
		NodeList nl = transitions.getChildNodes();
		int len = nl.getLength();
		for(int i = 0; i < len; i++) {
			Node transition = nl.item(i);
			short ft = transition.getNodeType();
			switch(ft) {
				case Node.ELEMENT_NODE:
					String fn = transition.getNodeName();
					if (fn.equals(JAM_GOTO_TAG) || fn.equals(JAM_CALL_TAG) || fn.equals(JAM_RETURN_TAG)) {
						addTransition((JspAppTransition)new JspAppTransition(this).parseNode(transition));
					}
					break;
				default:		
					break;
			}
		}
	}
}
