/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.oc.forms;


import org.opencms.file.*;
import org.opencms.main.*;
import org.w3c.dom.*;

import com.codedroids.forms.*;
import com.codedroids.oc.OCCache;
import com.codedroids.oc.OCCachedXmlDocument;
import org.opencms.jsp.CmsJspBean;

// deprecated stuff needs these
import java.util.*;
import java.io.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import com.codedroids.oc.OCUtils;

public class OCForm extends Form {
//	 deprecated stuff needs these
	private static Map formsCache;
	private static String CACHE_TIMESTAMP = "timestamp";
	private static String CACHE_INSTANCE = "instance";
	static {
		formsCache = new HashMap();
	}
	
	
	/**
	 * Returns a new Form instance from a possibly cached (in application scope) XML file.
	 * 
	 * <p>The XML Document for the form is actually taken from the application OCCache, but each instance is
	 * newly instantiated and parsed from the cached document. This is necessary as the Form object carries the data,
	 * furthermore each instance is set with the locale of the request context.
	 * 
	 * @param cmso
	 * @param xmlfile
	 * @return
	 */
	public static synchronized OCForm getInstance(CmsJspBean cms, String xmlfile) throws CmsException
	{
		Element root = OCCachedXmlDocument.getCachedXmlDocument(OCCache.getAppInstance(cms), cms, xmlfile).getDocumentElement();
		OCForm retval = new OCForm(cms.getCmsObject());
		retval.parseForm(root);
		retval.setLocale(cms.getRequestContext().getLocale());
		return retval;
	}


	/**
	 * @deprecated Use {@link getInstance(CmsJspBean cms, String xmlfile)}
	 */
  	public static synchronized OCForm getInstance(CmsObject cms, String xmlfile) throws ParserConfigurationException, SAXException, IOException, CmsException
	{
		// Currently caches the document tree rather than OCForm instances 
		// (which would have to be cloned as the instance is not immutable)
		Element root = null;
		Map m = (Map)formsCache.get(xmlfile);
		if(m != null) {
			long timestamp = ((Date)m.get(CACHE_TIMESTAMP)).getTime();
			CmsResource f = cms.readResource(xmlfile);
			if(f != null && f.getDateLastModified() <= timestamp)
				root = (Element)m.get(CACHE_INSTANCE);
		}
		if(root == null) {
			root = OCUtils.loadXMLFileFromVFS(cms, xmlfile).getDocumentElement();
			m = new HashMap();
			m.put(CACHE_INSTANCE, root);
			m.put(CACHE_TIMESTAMP, new Date());
			formsCache.put(xmlfile, m);
		}
		OCForm retval = new OCForm(cms);
		retval.parseForm(root);
		return retval;
	}

	// 
	// constructors
	//
	
	/**
	 * 
	 */
	public OCForm(CmsObject cms) {
		super();
		// setup the more renders
		setFieldRender("file-selector", new OCFileSelectorInputFieldRender(cms));
		setFieldRender("date-selector", new OCDateSelectorInputFieldRender(cms));
	}
}
