/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.records;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

abstract public class A_GroupClause extends A_Clause {

	List operands;
	
	abstract public String divider();
	
	public A_GroupClause() {
		super();
		init(null);
	}
	
	public A_GroupClause(List operands) {
		super();
		init(operands);
	}

	public void init(List operands) {
		if(operands != null)
			this.operands = new ArrayList(operands);
		else
			this.operands = new ArrayList();
	}
	
	// for incremental building the clause
	public void addClause(A_Clause c) {
		operands.add(c);
	}
	public void addClauses(List c) {
		operands.addAll(c);
	}
	
	public int cardinality() {
		Iterator i = operands.iterator();
		int c = 0;
		while(i.hasNext()) {
			c += ((A_Clause)i.next()).cardinality();
		}
		return c;
	}
	
	public Object getValue() {
		return getValues().get(0);
	}
	
	public List getValues() {
		Iterator i = operands.iterator();
		List all = new ArrayList();
		while(i.hasNext()) {
			A_Clause c = (A_Clause)i.next();
			if(c.cardinality()==1)
				all.add(c.getValue());
			else if(c.cardinality() > 1)
				all.addAll(c.getValues());
		}
		return all;
	}

	public RecordField getField() {
		return (RecordField)getFields().get(0);
	}
	
	public List getFields() {
		Iterator i = operands.iterator();
		List all = new ArrayList();
		while(i.hasNext()) {
			A_Clause c = (A_Clause)i.next();
			if(c.cardinality()==1)
				all.add(c.getField());
			else if(c.cardinality() > 1)
				all.addAll(c.getFields());
		}
		return all;
	}
	
	public String toSQL() {
		StringBuffer buf = new StringBuffer();
		if(operands.size() > 0) {
			buf.append("( ");
			Iterator i = operands.iterator();
			while(i.hasNext()) {
				A_Clause c = (A_Clause)i.next();
				buf.append(c.toSQL());
				if(i.hasNext())
					buf.append(divider());
			}
			buf.append(" )");
		}
		return buf.toString();
	}
	
}
